import contextlib
import unittest
import io
import re
import unittest.mock
from unittest import TextTestRunner

class MainTest(unittest.TestCase):

    def test_answer(self):
        capturedOutput = io.StringIO()                 
        sys.stdout = capturedOutput                     
        main()
        sys.stdout = sys.__stdout__ 

        checkQuery = [
            'PAN AMS CAS NYC HEL',
            'PAN AMS CAS HEL NYC',
            'PAN AMS NYC CAS HEL',
            'PAN AMS NYC HEL CAS',
            'PAN AMS HEL CAS NYC',
            'PAN AMS HEL NYC CAS',
            'PAN CAS AMS NYC HEL',
            'PAN CAS AMS HEL NYC',
            'PAN CAS NYC AMS HEL',
            'PAN CAS NYC HEL AMS',
            'PAN CAS HEL AMS NYC',
            'PAN CAS HEL NYC AMS',
            'PAN NYC AMS CAS HEL',
            'PAN NYC AMS HEL CAS',
            'PAN NYC CAS AMS HEL',
            'PAN NYC CAS HEL AMS',
            'PAN NYC HEL AMS CAS',
            'PAN NYC HEL CAS AMS',
            'PAN HEL AMS CAS NYC',
            'PAN HEL AMS NYC CAS',
            'PAN HEL CAS AMS NYC',
            'PAN HEL CAS NYC AMS',
            'PAN HEL NYC AMS CAS',
            'PAN HEL NYC CAS AMS'
        ]
        modelAnswerLength = len(checkQuery)


        output = list(capturedOutput.getvalue().strip().split("\n"))
        if output[-1] == "":
            output.pop()

        checkBool = True
        for n in output:
            if n not in checkQuery:
                checkBool = False
            else:
                checkQuery.remove(n)

        if len(checkQuery) > 0:
            checkBool = False
 
        self.assertNotEqual(len(output), 120, "You are printing all possible route combinations. That's not what's expected.")

        containsDigit = bool(re.search(r'\d', capturedOutput.getvalue()))
        self.assertFalse(containsDigit, "The output contains numbers. Remember to print the portnames!")

        startsWithPan = bool(all(route[:3] == 'PAN' for route in output))
        self.assertTrue(startsWithPan, "Remember to start the route with Panama!")

        self.assertFalse(len(output) > modelAnswerLength, "Too many route combinations. Are you missing some constraints?")
        self.assertFalse(len(output) < modelAnswerLength, "Too few route combinations. Are you missing some routes?")

        self.assertEqual(checkBool, True, "Your answer does not contain one of every possible route combinations or your list contains duplicates.")

        # Previous
        # self.assertEqual(capturedOutput.getvalue().strip(), 'PAN AMS CAS NYC HEL\nPAN AMS CAS HEL NYC\nPAN AMS NYC CAS HEL\nPAN AMS NYC HEL CAS\nPAN AMS HEL CAS NYC\nPAN AMS HEL NYC CAS\nPAN CAS AMS NYC HEL\nPAN CAS AMS HEL NYC\nPAN CAS NYC AMS HEL\nPAN CAS NYC HEL AMS\nPAN CAS HEL AMS NYC\nPAN CAS HEL NYC AMS\nPAN NYC AMS CAS HEL\nPAN NYC AMS HEL CAS\nPAN NYC CAS AMS HEL\nPAN NYC CAS HEL AMS\nPAN NYC HEL AMS CAS\nPAN NYC HEL CAS AMS\nPAN HEL AMS CAS NYC\nPAN HEL AMS NYC CAS\nPAN HEL CAS AMS NYC\nPAN HEL CAS NYC AMS\nPAN HEL NYC AMS CAS\nPAN HEL NYC CAS AMS')


test_suite = unittest.TestLoader().loadTestsFromTestCase(MainTest)
with io.StringIO() as buf:
    # run the tests
    with contextlib.redirect_stdout(buf):
        TextTestRunner(stream=buf).run(test_suite)
    # process (in this case: print) the results
    printTest(buf.getvalue())